/*
###########################################################################
# trustyRC, fully modular IRC robot 
# Copyright (C) 2006-2008 Nicoleau Fabien
#
# trustyRC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################
*/

/** @file syslog.cpp
 * @brief SysLog implementation file
 */

#include "syslog.h"

/**
 * Class constructor
 * Initialize private attributes
 * @param level log level 
 * @param vb true to display event in shell, else false
 * @param max Max log file size (in MB)
 * @param fname Log file name
 * @param bu 1 for backup logfile, else 0
 */
SysLog::SysLog(log_level level,bool vb,int max,string fname,string bu)
{
	this->setLogLevel(level);
	this->setVerbose(vb);
	this->setFile(fname);
	this->setMaxSize(max);
	this->setBackup(bu);
}

/**
 * Class constructor
 * Initialize private attributes
 * @param level log level 
 * @param vb true to display event in shell, else false
 * @param max Max log file size (in MB)
 * @param fname Log file name
 * @param bu 1 for backup logfile, else 0
 */
SysLog::SysLog(string level,bool vb,int max,string fname,string bu)
{
	this->setVerbose(vb);
	this->setLogLevel(level);
	this->setFile(fname);
	this->setMaxSize(max);
	this->setBackup(bu);
   
}

/**
 * Class destructor
 */
SysLog::~SysLog()
{
   delete this->myStream;
}

/**
 * Open the logfile
 * @return true if open OK, else false
 */
bool SysLog::open() {
   this->myStream = new ofstream();
   this->myStream->open(this->file.c_str(), std::ios_base::app);
   return !this->myStream->fail();
}

/**
 * Close the logfile
 * @return true if close OK, else false
 */
bool SysLog::close() {
   this->myStream->close();
   return !this->myStream->fail();
}

/**
 * Turn on/off verbose
 * @param vb true for On, false for Off
 */
void SysLog::setVerbose(bool vb)
{
	this->verbose = vb;
}

/**
 * Set log file name
 * @name File name
 */
void SysLog::setFile(string name)
{
	this->file = name;
}

/**
 * Log an event according to its level
 * If max log file size is attempted, it's truncated
 * if backup value is "1", a file backup is made
 * @param lvl event level
 * @param str event detail
 */
void SysLog::log(log_level lvl,string str)
{
	time_t now ;
   time(&now);
   struct stat stats;
	char timeFormat[17];
	strftime(timeFormat,18,"%y-%m-%d %X",localtime(&now));
	string logStr(timeFormat);
	logStr += " ";
	switch (lvl)
	{
		case INFO: logStr += "{I} " + str ;
		break;
		case WARNING: logStr += "{W} " + str  ;
		break;
		case ERROR: logStr += "{E} " + str ;
		break;
		default:	logStr += "{U} " + str ;
	}
	if ( this->verbose)
		cout << logStr << endl;
   switch (this->ll)
   {
   	case NONE:;
   	break;
   	case INFO: 	if (lvl == INFO ) {
                     *this->myStream << logStr << endl;
   					}
   	break;
   	case WARNING:if(lvl==INFO||lvl==WARNING) {
   						*this->myStream << logStr << endl;
   					}
   	break;
   	case ERROR:	*this->myStream << logStr << endl;
   	break;
   	default:	;
   }
   if (stat(this->file.c_str(),&stats)==0) {
	   if (stats.st_size > (this->maxSize*1024*1024) ) {//  (1024*1024 = MEGA) 
         this->close();
		   if (this->backup == "0" ) {
			   remove(this->file.c_str());	
		   }
		   else {
			   rename(this->file.c_str(),(this->file+".bak").c_str());
		   }
         delete this->myStream;
         this->open();
	   }
   }
   else {
      this->close();
      delete this->myStream ;
      this->open();
   }
}

/**
 * Set max log file size
 * @param nb sie (in MB)
 */
void SysLog::setMaxSize(int nb)
{
	this->maxSize = nb;
}

/**
 * Set log level 
 * @param level log level
 */
void SysLog::setLogLevel(log_level level)
{
	this->ll = level;
}

/**
 * Set log level 
 * @param level log level
 */
void SysLog::setLogLevel(string level)
{
	if(level=="error")
		this->setLogLevel(ERROR);
	else if (level=="warning")
		this->setLogLevel(WARNING);
	else if (level=="info")
		this->setLogLevel(INFO);
	else if (level=="none")
		this->setLogLevel(NONE);
	else
	{
		this->setLogLevel(ERROR);
		this->log(ERROR,"Unknown level : "+level);
		this->setLogLevel(NONE);
	}
}

/**
 * Set on/off for backup log file
 * @param bu "1" for on, "0" for off
 */
void SysLog::setBackup(string bu)
{
	this->backup = bu;
}

